@@ -1,5 +1,6 @@
 __author__ = 'ardevelop'
 
+import ujson
 import leveldb
 import tornado.web
 import tornado.ioloop
@@ -11,13 +12,15 @@ class Application(tornado.web.Application):
         self.get = 0
         self.miss = 0
         self.put = 0
-        super(Application, self).__init__([("/", StatsHandler), ("/(.+)", RequestHandler)], auto_etag=False)
+        super(Application, self).__init__(
+            [("/(.+)", MultipleRequestHandler), ("/(.+)", KeyRequestHandler), ("/stats", StatsHandler)],
+            auto_etag=False)
 
     def log_request(self, handler):
         pass
 
 
-class RequestHandler(tornado.web.RequestHandler):
+class KeyRequestHandler(tornado.web.RequestHandler):
     def get(self, key):
         app = self.application
         try:
@@ -41,6 +44,33 @@ class RequestHandler(tornado.web.RequestHandler):
         self.finish()
 
 
+class MultipleRequestHandler(tornado.web.RequestHandler):
+    def get(self):
+        app = self.application
+        keys = ujson.loads(self.request.body)
+        result = []
+        for key in keys:
+            try:
+                result.append(app.db.Get(key))
+            except KeyError:
+                result.append(None)
+
+        self.finish(ujson.dumps(result))
+
+    def put(self):
+        app = self.application
+        try:
+            mapping = ujson.loads(self.request.body)
+            for key, value in mapping.iteritems():
+                app.db.Put(key, value)
+
+        except (ValueError, AttributeError):
+            raise tornado.web.HTTPError(400)
+
+    def write_error(self, status_code, **kwargs):
+        self.finish()
+
+
 class StatsHandler(tornado.web.RequestHandler):
     def get(self):
         app = self.application
